#coding:utf8
import unittest

DEBUG = True

class Player():
    def __init__(self):
        self.avatar = ''
    def put_chess(self, location, board):
        c = Chess(location, self)
        if board.put_chess(c):
            return c
        else:
            return None
    def my_chesses(self, board):
        return board.get_player_chesses(self)

class ChessBoard():
    def __init__(self, x, y):
        self._chesses = []
        self.load_chess_board(x, y)
    def load_chess_board(self, x, y):
        self._chesses = []
        id = 0
        for yy in range(y):
            for xx in range(x):
                self._chesses.append(Position(id, xx, yy))
                id = id + 1
        return self._chesses
    def is_full(self):
        for c in self._chesses:
            if c.content == None:
                return False
        return True
    def put_chess(self, chess):
        for p in self._chesses:
            if p.id == chess.id and p.content is None:
                p.content = chess
                return True
        return False
    def get_chess(self, id):
        chess = None
        for p in self._chesses:
            if p.id == id and p.content is not None:
                chess = p
        return chess
    def get_player_chesses(self, player):
        chesses = []
        for p in self._chesses:
            if p.content and p.content.owner == player:
                chesses.append(p.content)
        return chesses
#     def chesses(self, id):
#         return self._chesses[id]
    
class Chess():
    def __init__(self, id, owner):
        self.id = id
        self.owner = owner

class Position():
    def __init__(self, id = -1, x = -1, y = -1):
        self.id = id
        self.x = x
        self.y = y
        self.content = None
    def XY(self):
        return (self.x, self.y)

class PositionTest(unittest.TestCase):
    def setUp(self):
        self.pos = Position()

class ChessBoardTest(unittest.TestCase):
    def setUp(self):
        self.chess_board = ChessBoard(3, 3)
    def testChessBoard(self):
        cb = ChessBoard(3, 3)
        chessboard = cb.load_chess_board(3, 3)
        self.assertTrue(isinstance(chessboard, list))
        self.assertTrue(isinstance(chessboard[0], Position))
        # Position(0, 0, 0) 
        self.assertTrue(0 == chessboard[0].id)
        self.assertTrue(0 == chessboard[0].x)
        self.assertTrue(0 == chessboard[0].y)
        # Position(1, 1, 0)
        self.assertTrue(1 == chessboard[1].id)
        self.assertTrue(1 == chessboard[1].x)
        self.assertTrue(0 == chessboard[1].y)
    def testBoardPutChess(self):
        chess1 = Chess(0, Player())
        self.assertTrue(self.chess_board.put_chess(chess1))
        self.assertFalse(self.chess_board.put_chess(chess1))
    def testGetChess(self):
        self.chess_board.put_chess(Chess(1, Player()))
        ch = self.chess_board.get_chess(1)
        self.assertTrue(1 == ch.id)
        ch2 = self.chess_board.get_chess(10)
        self.assertTrue(None == ch2)
    def testGetPlayerChesses(self):
        p = Player()
        self.chess_board.put_chess(Chess(1, p))
        chesses = self.chess_board.get_player_chesses(p)
        self.assertTrue(isinstance(chesses, list))
        self.assertTrue(chesses[0].owner == p)
        self.assertFalse(chesses[0].owner == Player())
    def testChessboardisFullOfChesses(self):
        c_board = ChessBoard(3, 3)
        self.assertFalse(c_board.is_full())
        for i in range(0, 9):
            c_board.put_chess(Chess(i, Player()))
        self.assertTrue(c_board.is_full())

class ChessTest(unittest.TestCase):
    def setUp(self):
        self.chess = Chess(0, Player())
    def testChessInfo(self):
        p = Player()
        c = Chess(0, p)
        self.assertTrue(p == c.owner)
        self.assertTrue(0 == c.id)
    def testChessOwner(self):
        p = Player()
        c = Chess(0, p)
        self.assertTrue(c.owner == p)
        self.assertFalse(c.owner == Player())

class PlayerTest(unittest.TestCase):
    def setUp(self):
        self.player = Player()
    def testPlayerPutChess(self):
        cb = ChessBoard(3,3)
        chess = self.player.put_chess(0, cb)
        self.assertTrue(isinstance(chess, Chess))
        self.assertTrue(0 == chess.id)
        chess = self.player.put_chess(0, cb)
        self.assertTrue(None == chess)
    def testPlayerAvatar(self):
        self.player.avatar = 'X'
        self.assertTrue('X' == self.player.avatar)
    def testPlayerChesses(self):
        cb = ChessBoard(3,3)
        self.player.put_chess(1, cb)
        chesses = self.player.my_chesses(cb)
        self.assertTrue(chesses[0].owner == self.player)

if __name__ == '__main__':
    if DEBUG:
        unittest.main()



#coding:utf8
import unittest
import itertools
from jinziqi_chess import Player, Chess, ChessBoard

DEBUG = False

class Jinziqi_core():
    def __init__(self, player_a_tag = 'A', player_b_tag = 'B'):
        self.players = []
        self.player_a = Player()
        self.player_a.avatar = player_a_tag
        self.player_b = Player()
        self.player_b.avatar = player_b_tag
        self.players.append(self.player_a)
        self.players.append(self.player_b)
        self.x = 3
        self.cb = ChessBoard(self.x, self.x)

    def print_chessboard(self):                         
        cbc = self.cb._chesses
        step = 1
        for c in cbc:
            if c.content == None:
                print c.id,
            else:
                print c.content.owner.avatar,
            if step == self.x:
                step = 1
                print ''
                continue
            step = step + 1
    def add_chess(self, player, location):
        return player.put_chess(location, self.cb)
    def exit_jinziqi(self, info):
        self.print_chessboard()
        who = info
        print "%s is win" % who
        print 'congratulation %s please run the program again' % who
        exit(0)
    def jinziqi_start2(self):
        '''
        input: player,location
        '''
        while(1):
            self.print_chessboard()
            str_in = input('enter player, location')
            player, location = str_in.split(',')
            location = int(location)
            if player == 'a':
                self.add_chess(self.player_a, location)
                if self.is_win([c.id for c in self.player_a.my_chesses(self.cb)]):
                    self.exit_jinziqi('p1')
            if player == 'b':
                self.add_chess(self.player_b, location)
                if self.is_win([c.id for c in self.player_b.my_chesses(self.cb)]):
                    self.exit_jinziqi('p2')
            if self.cb.is_full():
                print 'no one win'
    def is_win(self, p):
        combinations = self.player_chesses_combinations(p)
        #cb = [self.chessboard(c) for c in combinations] # 有坐标的棋子集合
        for chesses in combinations:
            a = self.chessboard(chesses[0])
            b = self.chessboard(chesses[1])
            c = self.chessboard(chesses[2])
            if self.isline(a, b, c):
                return True
        return False
    def get_chessboard(self):
        return self.cb
    def player_chesses_combinations(self, player, c = 3 ):
        return tuple(itertools.combinations(player, c))
    def chessboard(self, t):
        _ch = self.cb.get_chess(t)
        convert_obj = None
        if _ch:
            convert_obj = (_ch.x, _ch.y)
        return convert_obj
    def isline(self, a, b, c):
        if self._isline_x(a,b,c):
            return True
        if self._isline_y(a,b,c):
            return True
        if self._isline_xy_1(a,b,c):
            return True
        if self._isline_xy_2(a,b,c):
            return True
        return False
    def _isline_x(self, a, b, c):
        if a[0] == b[0] :
            if b[0] == c[0]:
                return True
            else:
                return False
        else:
            return False
    def _isline_y(self, a, b, c):
        if a[1] == b[1] :
            if b[1] == c[1]:
                return True
            else:
                return False
        else:
            return False
    def _isline_xy_1(self, a, b, c):
        #y=x
        c = [a,b,c]
        result = True
        for x,y in c:
            if x==y:
                result = True
            else:
                result = False
                break
        return result
    def _isline_xy_2(self, a, b, c):
        #y = 2-x
        c = [a,b,c]
        result = True
        for x,y in c:
            if y == (2-x):
                result = True
            else:
                result = False
                break
        return result

class JinziqiTest(unittest.TestCase):
    def setUp(self):
        self.jinziqi = Jinziqi_core()
    def testPlayerCount(self):
        '''only 2 player
        '''
        self.assertTrue(len(self.jinziqi.players) == 2)

    def test_player(self):
        '''测试玩家
        '''
        jinziqi_program = Jinziqi_core()
        self.assertEquals([],jinziqi_program.player_a.my_chesses(jinziqi_program.cb))
        self.assertEquals([],jinziqi_program.player_b.my_chesses(jinziqi_program.cb))

    def test_line_x(self):
        '''测试坐标是否符合一条横直线
        '''
        jinziqi_program = Jinziqi_core()
        chessboard = jinziqi_program.get_chessboard()
        chessboard.put_chess(Chess(0, Player()))
        a = jinziqi_program.chessboard(0)
        chessboard.put_chess(Chess(1, Player()))
        b = jinziqi_program.chessboard(1)
        chessboard.put_chess(Chess(2, Player()))
        c = jinziqi_program.chessboard(2)
        self.assertTrue(jinziqi_program.isline(a, b, c))
        chessboard.put_chess(Chess(1, Player()))
        a = jinziqi_program.chessboard(1)
        chessboard.put_chess(Chess(8, Player()))
        b = jinziqi_program.chessboard(8)
        chessboard.put_chess(Chess(3, Player()))
        c = jinziqi_program.chessboard(3)
        self.assertFalse(jinziqi_program.isline(a, b, c))
    def test_line_y(self):
        '''测试坐标是否符合一条纵直线
        '''
        jinziqi_program = Jinziqi_core()
        chessboard = jinziqi_program.get_chessboard()
        chessboard = jinziqi_program.get_chessboard()
        chessboard.put_chess(Chess(1, Player()))
        chessboard.put_chess(Chess(4, Player()))
        chessboard.put_chess(Chess(7, Player()))
        a = jinziqi_program.chessboard(1)
        b = jinziqi_program.chessboard(4)
        c = jinziqi_program.chessboard(7)
        self.assertTrue(jinziqi_program.isline(a, b, c))
    def test_line_xy(self):
        '''测试方程y=x和y=2-x两条直线
        '''
        jinziqi_program = Jinziqi_core()
        chessboard = jinziqi_program.get_chessboard()
        chessboard.put_chess(Chess(0, Player()))
        chessboard.put_chess(Chess(4, Player()))
        chessboard.put_chess(Chess(8, Player()))
        a = jinziqi_program.chessboard(0)
        b = jinziqi_program.chessboard(4)
        c = jinziqi_program.chessboard(8)
        self.assertTrue(jinziqi_program.isline(a, b, c))
        chessboard.put_chess(Chess(6, Player()))
        chessboard.put_chess(Chess(4, Player()))
        chessboard.put_chess(Chess(2, Player()))
        a = jinziqi_program.chessboard(6)
        b = jinziqi_program.chessboard(4)
        c = jinziqi_program.chessboard(2)
        self.assertTrue(jinziqi_program.isline(a, b, c))
    def test_chesses_combinations(self):
        '''测试棋子的组合
        '''
        jinziqi_program = Jinziqi_core()
        a = jinziqi_program.player_chesses_combinations([1,2,3,4,5])
        b = tuple(itertools.combinations([1,2,3,4,5], 3))
        self.assertTrue(b == a)
    def test_win(self):
        '''测试检测获胜的判断
        '''
        jinziqi_program = Jinziqi_core()
        chessboard = jinziqi_program.get_chessboard()
        chessboard.put_chess(Chess(0, Player()))
        chessboard.put_chess(Chess(1, Player()))
        chessboard.put_chess(Chess(2, Player()))
        chessboard.put_chess(Chess(6, Player()))
        chessboard.put_chess(Chess(7, Player()))
        self.assertTrue(jinziqi_program.is_win([0,1,2,6,7]))
    def test_add_chess(self):
        jinziqi_program = Jinziqi_core('X', 'O')
        p1 = Player()
        chess = jinziqi_program.add_chess(p1, 1)
        self.assertTrue(chess.owner == p1)
        self.assertTrue(chess.id == 1)
    def test_print_chessboard(self):
        jinziqi_program = Jinziqi_core('X', 'O')
        position = 7
        play_a = [1,2]
        play_b = [3,7]
        self.assertTrue('O' == jinziqi_program.print_chess(position, play_a, play_b))
        position = 7
        play_a = [1,7]
        play_b = [3,6]
        self.assertTrue('X' == jinziqi_program.print_chess(position, play_a, play_b))
        position = 5
        play_a = [1,2]
        play_b = [3,7]
        self.assertTrue('5' == jinziqi_program.print_chess(position, play_a, play_b))


if __name__ == '__main__':
    if DEBUG:
        unittest.main()
    else:
        jinziqi_program = Jinziqi_core()
        jinziqi_program.jinziqi_start2()




#coding:utf-8
from setuptools import setup, find_packages

setup(name='jinziqi',
      version=':versiontools:jinziqi:',
      description="jinziqi for python",
      long_description="""""",
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      author='samleung',
      author_email='callsamleung@gmail.com',
      url='https://github.com/callsamleung/jinziqi_python',
      license='mit',
      packages=find_packages(exclude=['.git*']),
      zip_safe=False,
      setup_requires=[
          'versiontools>=1.8',
          ],
      install_requires=[
          # -*- Extra requirements: -*-
      ],
      )
